#!/usr/bin/env python

# have_I_b33n_pwned.py
# Check if your password (hash) appears in the leaked password database
# of haveibeenpwned.com
#
# Copyright (C) 2019 willipink.eu
# Author Moritz Münch moritzmuench@mailbox.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
# updater script for nextcloud
#
# TODO
# - add feature: keepass integration? isnt there something like this already?


from os import name
from sys import argv, stdout, exit
from argparse import ArgumentParser
from hashlib import sha1
from getpass import getpass
from requests import get


if name == 'nt':
    (RED, GREEN, RESET) = ("", "", "")
else:
    RED   = "\033[1;31m"
    GREEN = "\033[0;32m"
    RESET = "\033[0;0m"
API   = 'https://api.pwnedpasswords.com/range/'
API_MAIL = 'https://haveibeenpwned.com/api/v2/breachedaccount/'
ROW   = '{:<30}{:<10}{:<45}'
HIDDEN = False


parser = ArgumentParser(
    description='Check if your email or password appears in a data leak.',
    epilog='Moritz Münch, moritzmuench@mailbox.org, willipink.eu, GPL3+'
)
parser.add_argument('-m', '-mail', '--mail',
    metavar='foo@adress.bar',
    nargs='+',
    help='Email address which should be checked against the leak database.'
)
parser.add_argument('-p', '-password', '--password',
    metavar='password',
    nargs='+',
    help='Password which should be checked against the leak database.'
)
args = parser.parse_args()
    


def header():
    print()
    print(ROW.format('password', 'leaked', 'sha1'))
    print('-' * 80)


def prompt_password_or_mail():
    print()
    while True:
        choice = input('Do you wannt to check a password [p] or an email adress [e]: ')
        if choice == 'p':
            prompt_password()
        elif choice == 'e':
            prompt_mail()


def prompt_password():
    print()
    password = getpass('Tell me your password: ')
    global HIDDEN
    HIDDEN = True
    header()
    query_password(password)


def prompt_mail():
    print()
    mail = input('Tell me your mail adress: ')
    global HIDDEN
    HIDDEN = True
    header()
    query_mail(mail)


def query_password(password):
    password_hash = sha1(password.encode('UTF-8')).hexdigest().upper()
    request = password_hash[:5]
    response = get(API + request).text
    hash_searched = 'not yet'
    for answer in response.splitlines():
        data = answer.split(':')
        combined_hash = request + data[0]
        if password_hash == combined_hash:
            hash_searched = int(data[1])
            break

    if hash_searched == 'not yet':
        stdout.write(GREEN)
    else:
        stdout.write(RED)

    if HIDDEN:
        password = '*' * len(password)

    print(ROW.format(password, hash_searched, password_hash))
    stdout.write(RESET)

    if HIDDEN:
        prompt_password_or_mail()


def query_mail(mail):
    print()
    try:
        response = get(API_MAIL + mail).json()
        for breach in response:
            print('Name: {}'.format(breach['Name']))
            print('Title: {}'.format(breach['Title']))
            print('Domain: {}'.format(breach['Domain']))
            print('Breach date: {}'.format(breach['BreachDate']))
            print('Added date: {}'.format(breach['AddedDate']))
            print('Modified date: {}'.format(breach['ModifiedDate']))
            print('Pwn count: {}'.format(breach['PwnCount']))
            print('Description: {}'.format(breach['Description']))
            print('Breached data: {}'.format(', '.join([data for data in breach['DataClasses']])))
            print('Verified: {}'.format('yes' if breach['IsVerified'] == 1 else 'No'))
            print('Fabricated: {}'.format('yes' if breach['IsFabricated'] == 1 else 'No'))
            print('Sensitive: {}'.format('yes' if breach['IsSensitive'] == 1 else 'No'))
            print('Retired: {}'.format('yes' if breach['IsRetired'] == 1 else 'No'))
            print('Spam: {}'.format('yes' if breach['IsSpamList'] == 1 else 'No'))
    except:
        print('The mail adress {} was not found in any leak databases so far'.format(mail))

    if HIDDEN:
        prompt_password_or_mail()


if __name__ == '__main__':
    if args.password:
        header()
        for password in args.password:
            query_password(password)
    elif args.mail:
        for mail in args.mail:
            query_mail(mail)
    else:
        prompt_password_or_mail()
    print()
    exit(0)
